@@ -5,6 +5,8 @@
 use crate::sanitize::{Sanitize, SanitizeError};
 use crate::secp256k1_instruction::verify_eth_addresses;
 use crate::{
+    ed25519_instruction::verify_signatures,
+    feature_set,
     hash::Hash,
     instruction::{CompiledInstruction, Instruction, InstructionError},
     message::Message,
@@ -18,6 +20,7 @@ use crate::{
     system_program,
 };
 use std::result;
+use std::sync::Arc;
 use thiserror::Error;
 
 /// Reasons a transaction might be rejected.
@@ -409,7 +412,7 @@ impl Transaction {
             .collect()
     }
 
-    pub fn verify_precompiles(&self, libsecp256k1_0_5_upgrade_enabled: bool) -> Result<()> {
+    pub fn verify_precompiles(&self, feature_set: &Arc<feature_set::FeatureSet>) -> Result<()> {
         for instruction in &self.message().instructions {
             // The Transaction may not be sanitized at this point
             if instruction.program_id_index as usize >= self.message().account_keys.len() {
@@ -427,9 +430,21 @@ impl Transaction {
                 let e = verify_eth_addresses(
                     data,
                     &instruction_datas,
-                    libsecp256k1_0_5_upgrade_enabled,
+                    feature_set.is_active(&feature_set::libsecp256k1_0_5_upgrade_enabled::id()),
                 );
                 e.map_err(|_| TransactionError::InvalidAccountIndex)?;
+            } else if crate::ed25519_program::check_id(program_id)
+                && feature_set.is_active(&feature_set::ed25519_program_enabled::id())
+            {
+                let instruction_datas: Vec<_> = self
+                    .message()
+                    .instructions
+                    .iter()
+                    .map(|instruction| instruction.data.as_ref())
+                    .collect();
+                let data = &instruction.data;
+                let e = verify_signatures(data, &instruction_datas);
+                e.map_err(|_| TransactionError::InvalidAccountIndex)?;
             }
         }
         Ok(())
